﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Linq;
using System.Text;
using System.Configuration;
using System.Runtime.Caching;
using System.Data;
using System.Data.SqlClient;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using EasyFast.Utility.CacheUtility;

namespace EasyFast.Repository
{
    public class ModelHelper<T>
    {
        private string prefix = ConfigurationManager.AppSettings["Prefix"];

        #region 表名、主键与TimeStamp
        //数据库表名称
        public string TableName
        {
            get
            {
                return prefix + typeof(T).Name; 
            }
            set
            {
                TableName = value;
            }
        }

        //表主键
        public string PrimaryKey
        {
            get
            {
                return GetPropertyInfo().First().Name;
            }
        }

        public string ModelGuid
        {
            get
            {
                return GetPropertyInfo()[1].Name;
            }
        }

        //TimeStamp键
        public string TimeStamp
        {
            get
            {
                return GetPropertyInfo().Last().Name;
            }
        }
        #endregion

        /// <summary>
        /// 从缓存中获取泛型类的PropertyInfo数组，如缓存失效，则创建新的缓存
        /// </summary>
        /// <returns></returns>
        public PropertyInfo[] GetPropertyInfo()
        {
            PropertyInfo[] infos;
            ObjectCache cache = MemoryCache.Default;
            string cacheKey = TableName + "_PropertyInfo";
            if (EasyFastCache.GetInstance().GetCache(cacheKey) == null)
            {
                infos = typeof(T).GetProperties();
                EasyFastCache.GetInstance().AddObjectForever(cacheKey, infos);
            }
            else
            {
                infos = (PropertyInfo[])EasyFastCache.GetInstance().GetCache(cacheKey);
            }
            return infos;
        }

        public SqlParameter GetSqlParameter(PropertyInfo pi,T model)
        {
            SqlParameter parameter;
            switch (pi.PropertyType.Name)
            {
                case "String":
                    int length = 0;
                    object[] attributes = pi.GetCustomAttributes(typeof(StringLengthAttribute), true);
                    if (attributes != null && attributes.Length > 0)
                    {
                        StringLengthAttribute myAttribute = attributes[0] as StringLengthAttribute;
                        if (myAttribute != null)
                        {
                            length = myAttribute.MaximumLength;
                        }
                    }
                    if (length != 0)
                    {
                        parameter = new SqlParameter("@" + pi.Name, SqlDbType.NVarChar, length);
                    }
                    else
                    {
                        parameter = new SqlParameter("@" + pi.Name, SqlDbType.NVarChar);
                    }
                    parameter.Value = pi.GetValue(model, null);
                    break;
                case "Int32":
                    parameter = new SqlParameter("@" + pi.Name, SqlDbType.Int, 4);
                    parameter.Value = pi.GetValue(model, null);
                    break;
                case "Int64":
                    parameter = new SqlParameter("@" + pi.Name, SqlDbType.BigInt, 8);
                    parameter.Value = pi.GetValue(model, null);
                    break;
                case "DataTime":
                    parameter = new SqlParameter("@" + pi.Name, SqlDbType.DateTime, 8);
                    parameter.Value = pi.GetValue(model, null);
                    break;
                case "Boolean":
                    parameter = new SqlParameter("@" + pi.Name, SqlDbType.Bit, 2);
                    parameter.Value = pi.GetValue(model, null);
                    break;
                case "Guid":
                    parameter = new SqlParameter("@" + pi.Name, SqlDbType.UniqueIdentifier, 16);
                    parameter.Value = pi.GetValue(model, null);
                    break;
                case "Byte":
                    parameter = new SqlParameter("@" + pi.Name, SqlDbType.Timestamp, 8);
                    parameter.Value = pi.GetValue(model, null);
                    break;
                default:
                    parameter = new SqlParameter("@" + pi.Name, pi.GetValue(model, null));
                    break;
            }
            return parameter;
        }
    }
}
