﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Runtime.Caching;

namespace EasyFast.Utility.CacheUtility
{
    public class EasyFastCache
    {
        private static object lockHelper = new object();
        private static EasyFastCache instance = null;

        private EasyFastCache() { }

        public static EasyFastCache GetInstance()
        {
            if (instance == null)
            {
                lock (lockHelper)
                {
                    if (instance == null)
                    {
                        instance = new EasyFastCache();
                    }
                }
            }
            return instance;
        }

        /// <summary>
        /// 添加一个对象到缓存中，如30分钟未访问，则删除该缓存项
        /// </summary>
        /// <param name="name">该缓存项的唯一标识符</param>
        /// <param name="obj">要插入的对象。</param>
        public void AddObject(string name, Object dt)
        {
            ObjectCache cache = MemoryCache.Default;
            CacheItemPolicy policy = new CacheItemPolicy();
            policy.SlidingExpiration = TimeSpan.FromMinutes(30);
            cache.Set(name, dt, policy);
        }

        /// <summary>
        /// 添加一个对象到缓存中，该对象拥有最高优先级，永不被删除
        /// </summary>
        /// <param name="name">该缓存项的唯一标识符</param>
        /// <param name="obj">要插入的对象。</param>
        public void AddObjectForever(string name, Object obj)
        {
            ObjectCache cache = MemoryCache.Default;
            CacheItemPolicy policy = new CacheItemPolicy();
            policy.Priority = CacheItemPriority.NotRemovable;
            cache.Set(name, obj, policy);
        }

        /// <summary>
        /// 添加一个DataTable到缓存中，并且依赖于数据库过期策略，该方法只支持SQL2005及以上
        /// </summary>
        /// <param name="name">该缓存项的唯一标识符</param>
        /// <param name="dt">要插入的DataTable</param>
        /// <param name="sm">数据库依赖监控对象</param>
        public void AddObject(string name, DataTable dt, SqlChangeMonitor sm)
        {
            ObjectCache cache = MemoryCache.Default;
            CacheItemPolicy policy = new CacheItemPolicy();
            policy.ChangeMonitors.Add(sm);
            cache.Set(name, dt, policy);
        }

        /// <summary>
        /// 获取指定标识符的缓存项
        /// </summary>
        /// <param name="cacheKey">该缓存项的唯一标识符</param>
        /// <returns>该缓存</returns>
        public object GetCache(string cacheKey)
        {
            ObjectCache cache = MemoryCache.Default;
            return cache[cacheKey];
        }
    }
}
